import 'dart:async';

import 'package:fc_components/util/adapt.dart';
import 'package:flutter/material.dart';

enum FcButtonSize {
  large, // 大
  normal, // 普通
  small, // 小
  mini // 迷你
}

class FcButton extends StatefulWidget {
  const FcButton({Key? key, 
    this.disabled = false,
    this.padding,
    this.margin,
    required this.onPressed,
    required this.child,
    this.color,
    this.borderColor,
    this.textStyle,
    this.size = FcButtonSize.normal,
    this.customSize,
    this.plain = false,
    this.round = false,
    this.radius,
    this.block = false,
  }) : super(key: key);

  final bool disabled; // 是够禁用
  final EdgeInsets? padding; // 自定义内边距
  final EdgeInsets? margin; // 按钮外边距
  final VoidCallback onPressed; // 点击事件
  final Widget child; // 子Widget
  final Color? color; // 按钮颜色（borderColor为null时也应用于与边框）
  final Color? borderColor; // 边框颜色（plain为trye时有效）
  final TextStyle? textStyle; // 自定义字体样式
  final FcButtonSize size; // 按钮尺寸
  final Size? customSize; // 按钮自定义尺寸
  final bool plain; // 是否朴素按钮
  final bool round; // 是否圆角按钮
  final double? radius; // 自定义圆角
  final bool block; // 块级按钮

  @override
  _FcButtonState createState() => _FcButtonState();
}

class _FcButtonState extends State<FcButton> {

  /// 不同尺寸下的按钮最小尺寸
  final Map<FcButtonSize, Size> buttonSizeGroup = {
    FcButtonSize.large: Size(Adapt.px(180), Adapt.px(92)),
    FcButtonSize.normal: Size(Adapt.px(168), Adapt.px(72)),
    FcButtonSize.small: Size(Adapt.px(158), Adapt.px(58)),
    FcButtonSize.mini: Size(Adapt.px(100), Adapt.px(40))
  };
  /// 不同尺寸下的按钮内边距
  final Map<FcButtonSize, EdgeInsets> buttonPaddingGroup = {
    FcButtonSize.large: EdgeInsets.symmetric(horizontal: Adapt.px(26)),
    FcButtonSize.normal: EdgeInsets.symmetric(horizontal: Adapt.px(26)),
    FcButtonSize.small: EdgeInsets.symmetric(horizontal: Adapt.px(26)),
    FcButtonSize.mini: EdgeInsets.symmetric(horizontal: Adapt.px(26))
  };
  /// 不同尺寸下的字体大小
  final Map<FcButtonSize, double> fontSizeGroup = {
    FcButtonSize.large: Adapt.px(29),
    FcButtonSize.normal: Adapt.px(26),
    FcButtonSize.small: Adapt.px(24),
    FcButtonSize.mini: Adapt.px(20)
  };
  /// 不同尺寸下的按钮圆角
  final Map<FcButtonSize, double> radiusGroup = {
    FcButtonSize.large: Adapt.px(20),
    FcButtonSize.normal: Adapt.px(20),
    FcButtonSize.small: Adapt.px(15),
    FcButtonSize.mini: Adapt.px(10)
  };

  /// 按钮颜色
  Color get color {
    if (widget.disabled) {
      return const Color(0xFFC0C0C0);
    }
    return (widget.color ?? const Color(0xFFD4052D));
  }
  /// 按钮边框颜色（pain为true时有效）
  Color get borderColor  {
    return (widget.borderColor ?? color);
  }
  /// 按钮具体最小尺寸
  Size get buttonSize {
    if (widget.customSize != null) {
      return widget.customSize!;
    }
    return buttonSizeGroup[widget.size]!;
  }
  /// 按钮内边距
  EdgeInsets get buttonPadding {
    if (widget.padding != null) {
      return widget.padding!;
    }
    return buttonPaddingGroup[widget.size]!;
  }
  /// 按钮字体大小
  double get fontSize {
    return fontSizeGroup[widget.size]!;
  }
  /// 按钮圆角
  double get radius {
    if (widget.radius != null) {
      return widget.radius!;
    }
    if (widget.round) {
      return 9999;
    }
    return radiusGroup[widget.size]!;
  }
  /// 字体样式
  TextStyle get textStyle {
    if (widget.textStyle != null) {
      return widget.textStyle!;
    }

    return TextStyle(
      color: widget.plain ? color : Colors.white,
      fontSize: fontSize,
      height: 1,
    );
  }

  bool onPress = false;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: widget.margin,
      child: InkWell(
        onTap: () {
          if (widget.disabled) {
            return;
          }

          widget.onPressed();
        },
        onHighlightChanged: (bool value) {
          if (onPress != value) {
            if (value) {
              setState(() {
                onPress = value;
              });
            } else {
              Timer(const Duration(milliseconds: 100), () {
                setState(() {
                  onPress = value;
                });
              });
            }
          }
        },
        child: Stack(
          children: [
            Container(
              padding: buttonPadding,
              decoration: widget.plain ? BoxDecoration(
                borderRadius: BorderRadius.all(Radius.circular(radius)),
                border: Border.all(color: borderColor)
              ) : BoxDecoration(
                color: color,
                borderRadius: BorderRadius.all(Radius.circular(radius))
              ),
              constraints: BoxConstraints(
                minWidth: buttonSize.width,
                minHeight: buttonSize.height
              ),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                mainAxisSize: widget.block ? MainAxisSize.max : MainAxisSize.min,
                children: [
                  DefaultTextStyle(
                    style: textStyle,
                    child: widget.child
                  )
                ]
              )
            ),
            Positioned(
              top: 0,
              bottom: 0,
              left: 0,
              right: 0,
              child: Container(
                decoration: BoxDecoration(
                    color: Colors.black.withOpacity(onPress ? 0.1 : 0),
                    borderRadius: BorderRadius.all(Radius.circular(radius))
                )
              )
            )
          ]
        )
      )
    );
  }
}